## File: bayes.R
## Author: John Rothfels (rothfels)
## Description: Bayesian Prediction
## --------------------------------

MidpointRule <- function(f, min, max, k, ...) {
  ## Numerically integrates a function using the midpoint rule.
  ## Args:
  ##   f - the function to integrate
  ##   min - low bound for integration
  ##   max - high bound for integration
  ##   k - the number of points to evaulate the function at
  ##   ... - any additional arguments to the function being integrated
}

## Put ComputeHeight as a separate function

## P(theta)
UPrior <- function(theta) {
  ## The unnormalized prior density function for the +1/-1 model. The valid
  ## range for theta is (0, 1) with the endpoints not included.
  ## Returns:
  ##   f(theta)
}

PriorNorm <- function(k) {
  ## Computes the normalizing constant for the prior density function.
  ## Args:
  ##   k - number of points to use for integration
}

Likelihood <- function(theta, x) {
  ## P(X (vector) | Theta)
  ## The likelihood function. This is the probability of seeing x given theta.
  ## Args:
  ##   theta - distribution parameter
  ##   x - the data points (+1/-1 values)
}

UPost <- function(theta, x) {
  ## The unnormalized posterior density.
  ## Args:
  ##   theta - prior distribution parameter
  ##   x - the data points (+1/-1 values)

  ## P(theta | X) = P(theta) * P(X | theta) / P(X)
}

Predict <- function(x, k) {
  ## Predicts the next data point. Passed the previous data points and the
  ## number of points to use for integrations using the midpoint rule.
  ## Integrates the prediction for the next data point given theta with respect
  ## to the unnormalized posterior distribution of theta. Since the posterior
  ## is unnormalized, this results in unnormalized probabilities of the next
  ## point being +1 and -1, which are normalized with respect to the total
  ## unnormalized probability.
  ##
  ## Args:
  ##   x - the data points (+1/-1 values)
  ##   k - the number of points to use for integration
  ## Returns:
  ##   A list with fields $prediction and $probability. The $prediction field
  ##   is the value (+1/-1) predicted for the next data point (whichever has
  ##   higher probability of being the next data point). The $probability field
  ##   is the probability given to that prediction.
}
